package com.muyu.enterprise.MQTT;


import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

/**
 * 客户端类
 * @Author：李庆帅
 * @Package：com.muyu.enterprise.mqtt
 * @Project：cloud-server
 * @name：MyMqttClient
 * @Date：2024/9/27 22:20
 */
public class MyMqttClient {

    public static MqttClient  mqttClient =null;
    private static MemoryPersistence memoryPersistence=null;
    private static MqttConnectOptions mqttConectOptions=null;
    private static String ClinentName=""; //待填	将在服务端出现的名字
    private static String IP=""; //待填  服务器IP


    public static void main(String[] args) {
        start(ClinentName);
    }



    public static void start(String clientId){
        //初始化连接设置对象
        mqttConectOptions=new MqttConnectOptions();
        //设置是否清空session,这里如果设置为false表示服务器会保留客户端的连接记录，
        //这里设置为true表示每次连接到服务器都以新的身份连接
        mqttConectOptions.setCleanSession(true);
        //设置连接超时时间，单位是秒
        mqttConectOptions.setConnectionTimeout(10);
        //设置持久化方式
        memoryPersistence=new MemoryPersistence();
        if(null!=clientId){
            try{
                mqttClient =new MqttClient("tcp://"+IP+":1883", clientId,memoryPersistence);
            } catch (Exception e) {
                // TODO 自动生成的捕获块
                throw new RuntimeException(e);
            }
        }
        System.out.println("连接状态："+mqttClient.isConnected());
        //设置连接和回调
        if(null!=mqttClient){
            if(!mqttClient.isConnected()){
                //创建回调函数对象
                MQTTReceiveCallback MQTTReceiveCallback = new MQTTReceiveCallback();
                //客户端添加回调函数
                mqttClient.setCallback(MQTTReceiveCallback);
                //创建连接
                try{
                    System.out.println("创建连接");
                    mqttClient.connect(mqttConectOptions);
                } catch (Exception e) {
                    // TODO 自动生成的捕获块
                    throw new RuntimeException(e);
                }
            }
        }else {
            System.out.println("mqttClient为空");
        }
        System.out.println("连接状态"+mqttClient.isConnected());
    }
    //	关闭连接
    public void closeConnect(){
        //关闭储存方式
        if(null!=memoryPersistence){
            try{
                memoryPersistence.close();
            } catch (Exception e) {
                // TODO 自动生成的捕获块
                throw new RuntimeException(e);
            }
        }else {
            System.out.println("memoryPersistence为空");
        }

        if(null!=mqttClient){
            if(mqttClient.isConnected()){
                try{
                    mqttClient.disconnect();
                    mqttClient.close();
                } catch (Exception e) {
                    // TODO 自动生成的捕获块
                    throw new RuntimeException(e);
                }
            }else {
                System.out.println("mqttClient未连接");
            }
        }else {
            System.out.println("mqttClient为空");
        }
    }


    //发布消息
    public static void publishMessage(String pubTopic,String message,int qos){
        if(null!=mqttClient && mqttClient.isConnected()){
            System.out.println("发布消息"+mqttClient.isConnected());
            System.out.println("id"+mqttClient.isConnected());
            MqttMessage mqttMessage = new MqttMessage();
            mqttMessage.setQos(qos);

            MqttTopic topic = mqttClient.getTopic(pubTopic);

            if(null!=topic){
                try{
                    MqttDeliveryToken publish = topic.publish(mqttMessage);
                    if(!publish.isComplete()){
                        System.out.println("消息发布成功");
                    }
                } catch (Exception e) {
                    // TODO 自动生成的捕获块
                    throw new RuntimeException(e);
                }
            }

        }
    }


    //重新连接
    public static void reConnect(){
        if(null!=mqttClient&&mqttClient.isConnected()){
            if(!mqttClient.isConnected()){
                if(null!=mqttConectOptions){
                    try{
                        mqttClient.connect(mqttConectOptions);
                    } catch (Exception e) {
                        // TODO 自动生成的捕获块
                        throw new RuntimeException(e);
                    }
                }else {
                    System.out.println("mqttConnectOptions为空");
                }
            }else {
                System.out.println("mqttClient为空或已连接");
            }
        }else {
            start(ClinentName);
        }
    }


    //订阅主题
    public static void suvTopic(String topic){
        if(null!=mqttClient && mqttClient.isConnected()){
            try{
                mqttClient.subscribe(topic,1);
            } catch (MqttException e) {
                // TODO 自动生成的捕获块
                throw new RuntimeException(e);
            }
        }else {
            System.out.println("mqttClient出错");
        }
    }


    //清空主题
    public void cleanTopic(String topic){
        if(null!=mqttClient && mqttClient.isConnected()){
            try{
                mqttClient.subscribe(topic);
            } catch (Exception e) {
                // TODO 自动生成的捕获块
                throw new RuntimeException(e);
            }
        }else {
            System.out.println("mqttClient出错");
        }
    }








}
