package com.muyu.enterprise.MQTT;

import lombok.extern.log4j.Log4j2;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

/**
 * 这是发送消息的服务端
 * 服务器向多个客户端推送主题，即不同客户端可向服务器订阅相同主题
 * @Author：李庆帅
 * @Package：com.muyu.enterprise.MQTT
 * @Project：cloud-server
 * @name：ServerMQTT
 * @Date：2024/9/28 14:32
 */
@Log4j2
public class ServerMQTT {

    //tcp://MQTT安装的服务器地址:MQTT定义的端口号
    public static final String HOST = "tcp://127.0.0.1:1883";
    //定义一个主题
    public static final String TOPIC = "pos_message_all";
    //定义MQTT的ID，可以在MQTT服务配置中指定
    private static final String clientId = "server11";

    private MqttClient client;
    private static MqttTopic topic11;

    //    private String userName = "mqtt";  //非必须
    //    private String passWord = "mqtt";  //非必须

    private static MqttMessage message;

    /**
     * 构造方法
     * @throws MqttException
     */
    public ServerMQTT() throws MqttException {
        // MemoryPersistence设置clientid的保存形式，默认为以内存保存
        client=new MqttClient(HOST, clientId,new MemoryPersistence());
        connect();
    }

    /**
     * 用来连接服务器
     */
    private void connect()
    {
        MqttConnectOptions options = new MqttConnectOptions();
        options.setCleanSession(false);
        //        options.setUserName(userName);
        //        options.setPassword(passWord.toCharArray());
        // 设置超时时间
        options.setConnectionTimeout(10);
        // 设置会话心跳时间
        options.setKeepAliveInterval(20);
        try{
            client.setCallback(new PushCallback());
            client.connect(options);
            topic11 = client.getTopic(TOPIC);

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     *
     * @param topic
     * @param message
     * @throws MqttException
     */
    public static void publish(MqttTopic topic , MqttMessage message) throws MqttPersistenceException,MqttException{
        MqttDeliveryToken  token= topic.publish(message);

        token.waitForCompletion();
        System.out.println("消息已完全发布！"+token.isComplete());
        log.info("消息已完全发布！"+token.isComplete());

    }

    /**
     *
     * @param clieId
     * @param msg
     * @throws Exception
     */
    public static void sendMessage(String clieId,String msg)throws Exception{
        ServerMQTT server = new ServerMQTT();
        server.message = new MqttMessage();
        server.message.setQos(1);  //保证消息能到达一次
        server.message.setRetained(true);
        String str="{\"clienId\":\""+clieId+"\",\"msg\":\""+msg+"\"}";
        try{
            publish(server.topic11 , server.message);
            //断开连接
//            server.client.disconnect();
        }catch (Exception e){
            e.printStackTrace();
        }
    }


    public static void main(String[] args) throws Exception {
        sendMessage("123444","哈哈哈");
    }





}
